#!/usr/bin/env python3
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import optimize_gamma
from colorspacious import cspace_converter, ciecam02
import fit_cmap_ceiling as fcc

hexbackground = '#d0d0d0'

#### Some extra defaults
scallight = 1
swwht = 1

#### Presets
#from preset_user import *
#from preset_purple2orange import *
#from preset_purple2yellow import *
#from preset_purple2white import *
from preset_xpurple2white import *
#from preset_xblue2white import *
#from preset_tempscale import *
#from preset_default import *
#from preset_cubedefault import *
#from preset_tint import *
#from preset_beach import *
#from preset_lagoon import *
#from preset_ocean import *
#from preset_furnace import *
#from preset_slime import *
#from preset_nebula0001 import *
#from preset_inverse_nebula import *

####initialise booleans
do_iter = False; write_gpmap = False; show_plots = False; show_test_image = False
trimWhite=True
gpsteps=False
#invmode=1; ginv=1.
#reverse=True

####basic controls
#do_iter = True
write_gpmap = True
show_plots = True
show_test_image = True

nlev = 1024
nsteps = 25
#gpsteps = True

gstart = gamma
gamit = 100
gameps = 1.e-8
rgbeps = 1.e-10
epsSat = 1.e-12

cfitmode = 1 #0:off, 1:fit for color gradient display, 2:fit for curves & output map, too

iclipmin = int(cliprange[0]*(nlev-1)+0.5)
iclipmax = int(cliprange[1]*(nlev-1)+0.5)
print('iclipmin = ', iclipmin)
print('iclipmax = ', iclipmax)

endcolor = start+360*rot
endcolor -= 360.*np.floor(endcolor/360.)
print('End color angle = ', endcolor)

cx = optimize_gamma.cmap(gamit=gamit, do_iter=do_iter, start=start, rot=rot, gstart=gstart, gamp=gamp, grot=grot,
                         ginv=ginv, pwr_amp=pwr_amp, eps_amp=eps_amp, eps_pwr=eps_pwr, sw_sc=sw_sc, pwr_sc=pwr_sc,
                         k_ellipse=k_ellipse, alpha_ellipse=alpha_ellipse, insectoid=insectoid,
                         reverse=reverse, invmode=invmode, cuberot=cuberot, nlev=nlev, minSat=minSat, maxSat=maxSat,
                         scalsat=scalsat, minLight=minLight, maxLight=maxLight, scallight=scallight,
                         ColorModel=ColorModel, iterSat=iterSat, epsSat=epsSat, rgbebs=rgbeps, gameps=gameps,
                         cliprange=cliprange, wluma=wluma, trimWhite=trimWhite, verbose=verbose,
                         tintMode=tintMode, tintGamma=tintGamma, startRGB=startRGB, endRGB=endRGB,
                         startC3=startC3, endC3=endC3, startPwr=startPwr, endPwr=endPwr, whtPwr=whtPwr, swwht=swwht)

if cfitmode == 2: cx = fcc.cmap(cx)

print('maxrgb = ', optimize_gamma.maxrgb_export)
print('minrgb = ', optimize_gamma.minrgb_export)

fract = np.linspace(0, 1, nlev)
def plot_testimage(cx, cfitmode):
    n = 1000000
    m = 100
    r = 2.5
    x = np.random.randn(n)
    y = np.random.randn(n)
    fig, ax = plt.subplots(figsize=(6, 6), constrained_layout=True)
    fig.patch.set_facecolor(hexbackground)
    ax.set_xlim([-r,r])
    ax.set_ylim([-r,r])
    ax.set_axis_off()
    if cfitmode == 1: cx = fcc.cmap(cx)
    gradient = np.vstack((fract, fract))
    plt.hexbin(x, y, gridsize=m, cmap=cx)
    plt.show(block=False)

def plot_combined(cx, nlevgrad, cfitmode, reverse, invmode):
    wluma=[0.299, 0.587, 0.114, 0]
    #wluma = [0.2126, 0.7152, 0.0722, 0]
    fract = np.linspace(0, 1, nlevgrad)
    rgba = cx(fract)
    srgb = cx(fract)[:,:3]
    Lab = cspace_converter("sRGB1", "CAM02-UCS")(srgb)
    L = Lab[:,0]
    a = Lab[:,1]
    b = Lab[:,2]
    C = np.sqrt(a**2+b**2)
    Y = np.dot(rgba, wluma)
    fig, axes = plt.subplots(figsize=(8, 8), nrows=2, constrained_layout=True)
    ax0 = axes[0]
    ax1 = axes[1]
    fig.patch.set_facecolor(hexbackground) #set window background
    ax0.set_facecolor(hexbackground) #set graph background
    ax0.plot(fract, 0.01*L, color='k', linestyle='--', alpha=0.5)
    ax0.plot(fract, Y, color='k', linestyle='-', alpha=1)
    if (reverse and invmode <= 0) or (not reverse and invmode > 0):
        ax0.plot(fract, 1-fract, color='k', linestyle=':', alpha=0.5)
    else:
        ax0.plot(fract, fract, color='k', linestyle=':', alpha=0.5)
    ax0.plot(fract, 0.01*C, color='#a08000', linestyle='--', alpha=1)
    #col = ['r', 'g', 'b']
    col = ['#a80000', '#008000', '#0000ff']
    for i in range(3):
        ax0.plot(fract, rgba[:, i], color=col[i])
    ax0.set_xlabel('index')
    ax0.set_ylabel('RGB')
    ax0.set_xlim([0,1])
    ax0.set_ylim([0,1])
    ax0.set_xticks(np.arange(0, 1, step=0.1))
    ax0.set_yticks(np.arange(0, 1, step=0.1))
    ax0.minorticks_on()
    if cfitmode == 1: cx = fcc.cmap(cx)
    gradient = np.vstack((fract, fract))
    ax1.imshow(gradient, aspect='auto', cmap=cx)
    #ax1.set_axis_off()
    ax1.get_xaxis().set_visible(False)
    ax1.get_yaxis().set_visible(False)
    ax0.grid(True, which='major', color='#808080')
    ax0.grid(True, which='minor', color='#b0b0b0')
    plt.show()

def plot_rgbhelix(cx, nlevgrad):
    fract = np.linspace(0, 1, nlevgrad)
    rgba = cx(fract)
    srgb = rgba[:,:3]
    rr = rgba[:,0]
    gg = rgba[:,1]
    bb = rgba[:,2]
    Lab = cspace_converter("sRGB1", "CAM02-UCS")(srgb)
    L = Lab[:,0]
    a = Lab[:,1]
    b = Lab[:,2]
    fig = plt.figure(figsize=(8, 8))
    ax = fig.gca(projection='3d')
    ax.set_xlabel('r')
    ax.set_ylabel('g')
    ax.set_zlabel('b')
    ax.set_xlim([0,1])
    ax.set_ylim([0,1])
    ax.set_zlim([0,1])
    ax.plot(rr, gg, bb)
    plt.show()
    
def plot_labhelix(cx, nlevgrad):
    fract = np.linspace(0, 1, nlevgrad)
    rgba = cx(fract)
    srgb = rgba[:,:3]
    rr = rgba[:,0]
    gg = rgba[:,1]
    bb = rgba[:,2]
    Lab = cspace_converter("sRGB1", "CAM02-UCS")(srgb)
    L = Lab[:,0]
    a = Lab[:,1]
    b = Lab[:,2]
    fig = plt.figure(figsize=(8, 8))
    ax = fig.gca(projection='3d')
    ax.set_xlabel('a')
    ax.set_ylabel('b')
    ax.set_zlabel('L')
    ax.set_xlim([-45,45])
    ax.set_ylim([-45,45])
    ax.set_zlim([0,100])
    ax.plot(a, b, L)
    plt.show()

if write_gpmap:
    fgp = open('x.rgba', 'w')
    fgp.write('# {:}\n'.format(cx.name))
    fgp.write('# gamit={:d}; do_iter={:}\n'.format(gamit, do_iter))
    fgp.write('# start={:.3f}; rot={:.6f}\n'.format(start, rot))
    fgp.write('# scalsat={:.3f}; gstart={:.3f}\n'.format(scalsat, gstart))
    fgp.write('# gamma={:.6f}; gamp={:.3f}; grot={:.3f}\n'.format(optimize_gamma.gamma_export, gamp, grot))
    fgp.write('# nlev={:d}; ColorModel={:d}; reverse={:}; cuberot={:}\n'.format(nlev, ColorModel, reverse, cuberot))
    fgp.write('# minLight={:.3f}; maxLight={:.3f}; scallight={:.3f}; trimWhite={:}\n'.format(minLight, maxLight, scallight, trimWhite))
    fgp.write('# pwr_amp={:.3f}; eps_amp={:.3f}; eps_pwr={:.3f}\n'.format(pwr_amp, eps_amp, eps_pwr))
    fgp.write('# sw_sc={:.3f}; pwr_sc={:.3f}\n'.format(sw_sc, pwr_sc))
    fgp.write('# k_ellipse={:.3f}; alpha_ellipse={:.3f}; insectoid={:}\n'.format(k_ellipse, alpha_ellipse, insectoid)) #k_ellipse=k_ellipse, alpha_ellipse=alpha_ellipse, insectoid=insectoid,
    fgp.write('# iterSat={:d}; epsSat={:.3e}; rgbeps={:.3e}; gameps={:.3e}\n'.format(iterSat, epsSat, rgbeps, gameps))
    fgp.write('# cliprange={:}; wluma={:}\n'.format(cliprange, wluma))
    if tintMode:
        fgp.write('# startRGB={:}; endRGB={:}\n'.format(startRGB, endRGB))
        fgp.write('# startPwr={:}; endPwr={:}\n'.format(startPwr, endPwr))
        fgp.write('# whtPwr={:}; swwht={:} \n'.format(whtPwr, swwht))
    if gpsteps:
        fgp.write('# gnuplot nsteps={:d}\n'.format(nsteps))
        for i in range(nsteps):
            x = i/(nsteps-1)
            x1 = i/nsteps
            outline = '{:.6f}  {:.6f} {:.6f} {:.6f} {:.6f}'.format(x1, *cx(x))
            fgp.write(outline+'\n')
            if i < nsteps:
                x2 = (i+1)/nsteps
                outline2 = '{:.6f}  {:.6f} {:.6f} {:.6f} {:.6f}'.format(x2, *cx(x))
                fgp.write(outline2+'\n')
    else:   
        for i in range(nlev):
            x = i/(nlev-1)
            outline = '{:.6f}  {:.6f} {:.6f} {:.6f} {:.6f}'.format(x, *cx(x))
            fgp.write(outline+'\n')
    fgp.close()

if show_plots or show_test_image:
    if gpsteps:
        nfract = nsteps
    else:
        nfract = nlev
    if show_test_image: plot_testimage(cx, cfitmode)
    if show_plots: plot_combined(cx, nfract, cfitmode, reverse, invmode)
    #plot_rgbhelix(cx, nfract)
    #plot_labhelix(cx, nfract)
