import numpy as np
import colorhelix

def cmap(gamit=100, do_iter=True, start=60, rot=-1.5, gstart=1.0, gamp=1.0, grot=1.0,
         pwr_amp=1.0, eps_amp=0.05, eps_pwr=2, sw_sc=0, pwr_sc=2,
         k_ellipse=0.0, alpha_ellipse=0.0, insectoid=False,
         reverse=False, cuberot=False, nlev=256, minSat=1.,
         maxSat=1., scalsat=1., minLight=0., maxLight=1., scallight=1.,
         ColorModel=2, iterSat=100, epsSat=1.e-12, rgbebs=1.e-10, gameps=1.e-8,
         cliprange=[0.1, 0.9], wluma=[0.3, 0.59, 0.11], trimWhite=True, verbose=0,
         **kwargs):
    global gamma_export, minrgb_export, maxrgb_export
    """
    Gamma-optimization for maximum hue.
    
    Parameters:
    gamit : integer, optional
        Maximum number of iterations. Defaults to 100
    do_iter : boolean, optional
        on/off switch. Defaults tu True
    rgbeps : scalar, optional
        Accuracy criterion for RGB components. Defaults to 1.e-10
    gameps : scalar, optional
        Accuracy criterion for gamma (must be >> rgbeps). Defaults to 1.e-8
    others : See colorhelix.py for details.
    """

    sgn = 0
    sgo = [1, -1]
    gameps = 1.e-8
    rgbeps = 1.e-10
    gamma = gstart
    dgamma = 0.1
    iclipmin = int(cliprange[0]*(nlev-1)+0.5)
    iclipmax = int(cliprange[1]*(nlev-1)+0.5)
    fract = np.linspace(0, 1, nlev)
    finished_iter = not do_iter
    finished_all = False
    
    minrgb = 0
    maxrgb = 1

    for i in range(gamit):
        if scallight < 0:
            if finished_all:
                scallight_use = 1/maxrgb
            else:
                scallight_use = 1
        else:
            scallight_use = scallight

        cx = colorhelix.cmap(start=start, rot=rot, gamma=gamma, gamp=gamp, grot=grot,
                             pwr_amp=pwr_amp, eps_amp=eps_amp, eps_pwr=eps_pwr, sw_sc=sw_sc, pwr_sc=pwr_sc,
                             k_ellipse=k_ellipse, alpha_ellipse=alpha_ellipse, insectoid=insectoid,
                             reverse=reverse, cuberot=cuberot, nlev=nlev, minSat=minSat, scallight=scallight_use,
                             maxSat=maxSat, scalsat=scalsat, minLight=minLight, maxLight=maxLight,
                             ColorModel=ColorModel, iterSat=iterSat, epsSat=epsSat, cliprange=cliprange,
                             wluma=wluma, trimWhite=trimWhite, verbose=verbose,
                             **kwargs)
        rgb = cx(fract)[:,:3]
        if finished_all: break
        minrgb = np.amin(rgb[iclipmin:iclipmax, :])
        maxrgb = np.amax(rgb[iclipmin:iclipmax, :])
        if finished_iter: finished_all = True
        if finished_iter: continue
        if minrgb > rgbeps:
            sgn = 1
        elif maxrgb < 1-rgbeps:
            sgn = -1
        if sgo[0] != sgn: #change of direction
            dgamma /= 2
        if verbose > 0:
            print('iter = {:2d}, gamma = {:.6f}, minrgb = {:.6f}, maxrgb = {:.6f}, sgn = {:2d}'
                  .format(i, gamma, minrgb, maxrgb, sgn))
        gamma += sgn*dgamma
        sgo[1] = sgo[0]
        sgo[0] = sgn
        finished_iter = dgamma < gameps or not do_iter

    print('')
    minrgb_export = minrgb
    maxrgb_export = maxrgb
    gamma_export = gamma
    print('gamma = {:.6f}'.format(gamma))
    if do_iter and (gamma < 0.5 or gamma > 2):
        print('')
        print('*** Solution for gamma value is quite extreme ***')
        print('')
    if do_iter: cx.name = cx.name+'_optgamma'
    return cx
