#!/usr/bin/env python3
import numpy as np
import matplotlib.pyplot as plt
import colorhelix
from colorspacious import cspace_converter

####defaults
from preset_default import *

plottype = 2
selection = 5

nlev=256
nlevgrad = 256
sat = 1.
scalsat = 1
cliprange = [0.1, 0.9]

tintMode = 1
rgb050015=[0.5, 0.0, 1.5]
rgb100015=[1.0, 0.0, 1.5]
startRGB = np.array([0.5, 0.0, 1.5]); startPwr = 8.0
endRGB = np.array([1.0, 1.0, 1.]); endPwr = 2.0
whtPwr = -4.0

## cubehelix settings
#wluma=[0.333, 0.333, 0.333]
wluma=[0.299, 0.587, 0.114]
#wluma = [0.2126, 0.7152, 0.0722]
#wluma = [0.1000, 0.8500, 0.0500]

cx0 = colorhelix.cmap(start=15, rot=-1.5, gamma=0.75, scalsat=scalsat, cliprange=cliprange)
#cx0 = colorhelix.cmap(start=15, rot=-1.5, gamma=0.75, scalsat=scalsat, cliprange=cliprange,
#                      tintMode=tintMode, startRGB=rgb050015, endRGB=endRGB, startPwr=startPwr, endPwr=endPwr,whtPwr=whtPwr)
#cx1 = colorhelix.cmap(start=15, rot=-1.5, gamma=0.75, scalsat=scalsat, cliprange=cliprange,
#                      tintMode=tintMode, startRGB=rgb100015, endRGB=endRGB, startPwr=startPwr, endPwr=endPwr,whtPwr=whtPwr)

cx1 = colorhelix.cmap(start=0, rot=-1, gamma=0.75, scalsat=scalsat, cliprange=cliprange)

cx2 = colorhelix.cmap(start=200, rot=-1, gamma=0.7, scalsat=scalsat, cliprange=cliprange)

#cx3 = colorhelix.cmap(start=165, rot=0.541667, gamma=0.8, scalsat=scalsat, cliprange=cliprange)
cx3 = colorhelix.cmap(start=270, rot=0.6, gamma=0.8, gamp=1.6, scalsat=scalsat, cliprange=cliprange)

#cx4 = colorhelix.cmap(start=290, rot=-0.25, gamma=0.7, scalsat=scalsat, cliprange=cliprange)
#cx4 = colorhelix.cmap(start=295, rot=-0.25, gamma=0.811, scalsat=scalsat, cliprange=cliprange)
cx4 = colorhelix.cmap(start=290, rot=-0.25, gamma=0.792, scalsat=scalsat, minLight=0.1, maxLight=1)
#cx4 = colorhelix.cmap(start=310, rot=-0.35, gamma=0.8, scalsat=scalsat, minLight=0.1, maxLight=1)
#cx4 = colorhelix.cmap(start=315, rot=-0.4, gamma=0.7, scalsat=scalsat, minLight=0.05, maxLight=1)

#cx4 = colorhelix.cmap(start=270, rot=0.541667, gamma=1., minLight=0, maxLight=1, ColorModel=2)
cx5 = colorhelix.cmap(start=270, rot=0.541667, gamma=0.85, minLight=0, maxLight=1)
#cx5 = colorhelix.cmap(start=270, rot=0.541667, gamma=0.828, minLight=0.1, maxLight=0.9)
#cx5 = colorhelix.cmap(start=270, rot=0.541667, gamma=0.84, minLight=0, maxLight=0.95)
#cx5 = colorhelix.cmap(start=270, rot=0.541667, gamma=0.854, minLight=0.05, maxLight=0.95)
#cx5 = colorhelix.cmap(start=270, rot=0.541667, gamma=1, sw_sc=-0.5, minLight=0.05, maxLight=0.95)
#cx5 = colorhelix.cmap(start=270, rot=0.541667, gamma=0.914, minLight=0.1, maxLight=1)

acx = [cx0, cx1, cx2, cx3, cx4, cx5]

#fract = np.linspace(0, 1, nlev)
#print('cx0 rgb = ',cx0(fract)[:,:3])
#print('cx5 max = ',cx0(0.5)[:3])

def plot_testimage(cx):
    n = 100000
    m = 100
    r = 3
    x = np.random.randn(n)
    y = np.random.randn(n)
    fig, ax = plt.subplots(figsize=(6, 6), constrained_layout=True)
    ax.set_xlim([-r,r])
    ax.set_ylim([-r,r])
    ax.set_axis_off()
    plt.hexbin(x, y, gridsize=m, cmap=cx)
    plt.show(block=False)

def plot_combined(cx, nlevgrad):
    #wluma=[0.299, 0.587, 0.114, 0]
    wluma = [0.2126, 0.7152, 0.0722, 0]
    fract = np.linspace(0, 1, nlevgrad)
    rgba = cx(fract)
    srgb = cx(fract)[:,:3]
    Lab = cspace_converter("sRGB1", "CAM02-UCS")(srgb)
    a = Lab[:,1]
    b = Lab[:,2]
    C = 0.01*np.sqrt(a**2+b**2)
    L = 0.01*Lab[:,0]
    Y = np.dot(rgba, wluma)
    fig, axes = plt.subplots(figsize=(8, 8), nrows=2, constrained_layout=True)
    ax0 = axes[0]
    ax1 = axes[1]
    ax0.plot(fract, L, color='k', linestyle='--', alpha=0.5)
    ax0.plot(fract, Y, color='k', linestyle='-', alpha=1)
    ax0.plot(fract, C, color='y', linestyle='--', alpha=1)
    ax0.plot(fract, fract, color='k', linestyle=':', alpha=0.5)
    col = ['r', 'g', 'b']
    #for xx in [0.25, 0.5, 0.75]:
    #    ax.axvline(xx, color='0.7', linestyle='--')
    for i in range(3):
        #ax0.plot(np.arange(nlevgrad)/nlevgrad, rgba[:, i], color=col[i])
        ax0.plot(fract, rgba[:, i], color=col[i])
    ax0.set_xlabel('index')
    ax0.set_ylabel('RGB')
    ax0.set_xlim([0,1])
    ax0.set_ylim([0,1])
    gradient = np.linspace(0, 1, nlevgrad)
    gradient = np.vstack((gradient, gradient))
    ax1.imshow(gradient, aspect='auto', cmap=cx)
    #ax1.set_axis_off()
    ax1.get_xaxis().set_visible(False)
    ax1.get_yaxis().set_visible(False)
    plt.show()

def plot_gradients(acx, nlevgrad):
    n = len(acx)
    gradient = np.linspace(0, 1, nlevgrad)
    gradient = np.vstack((gradient, gradient))
    fig, axes = plt.subplots(figsize=(8, 8), nrows=n, constrained_layout=True)
    for i in range(n):
        ax = axes[i]
        ax.imshow(gradient, aspect='auto', cmap=acx[i])
        ax.get_xaxis().set_visible(False)
        ax.get_yaxis().set_visible(False)
    plt.show()

if plottype == 1:
    plot_testimage(acx[selection])
    plot_combined(acx[selection], nlevgrad)
elif plottype == 2:
    plot_gradients(acx, nlevgrad)
elif -1 <= plottype <= 0:
    cx = acx[selection]
    if plottype == 0: fgp = open('colorhelix.gpmap', 'w')
    for i in range(nlev):
        x = i/(nlev-1)
        #print(x, cx(x))
        outline = '{:.6f}  {:.6f} {:.6f} {:.6f} {:.6f}'.format(x, *cx(x))
        print(outline)
        if plottype == 0: fgp.write(outline+'\n')
    if plottype == 0: fgp.close()
