#!/usr/bin/env python3
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import optimize_gamma
from colorspacious import cspace_converter, ciecam02
import fit_cmap_ceiling as fcc
####initialise booleans
do_iter = False; write_gpmap = False; show_test_image = False

####defaults
gamp=1.0; grot=1.0; pwr_amp=1.0; eps_amp=0.05; eps_pwr=2; sw_sc=0; pwr_sc=2
k_ellipse=0.0; alpha_ellipse=0.0; insectoid=False; reverse=False; cuberot=False
nlev=256; minSat=1.; maxSat=1.; scalsat=1.; minLight=0.; maxLight=1.
ColorModel=2; iterSat=100; epsSat=1.e-12; rgbebs=1.e-10; gameps=1.e-8
cliprange=[0.1, 0.9]; wluma=[0.3, 0.59, 0.11]; trimWhite=False
verbose=1
#kwargs defaults
tintMode = 0
startRGB = [1.0, 1.0, 1.0]; startPwr = 2.0
endRGB = [1.0, 1.0, 1.0]; endPwr = 2.0
whtPwr = 4.0

####basic controls
#do_iter = True
write_gpmap = True
show_test_image = True

nlev=256

####iteration settings
#gstart = 0.816843
#gstart = 0.5556 #for slime
gstart = 0.75
gamit = 100
gameps = 1.e-8
rgbeps = 1.e-10
epsSat=1.e-12

####examples
start=15; rot=-1.5#; gamp=0.5
start=0; rot=-1
#start=0; rot=-0.55; gamp=1. #ocean
#start=60; rot=0.5#; pwr_amp=4.; eps_amp=0.05; eps_pwr=2 #for slime
#start=270; rot=0.541667
#start=270; rot=0.6; gamp=1.
#start=270; rot=0.6; gamp=1.062 #maxhue purple fire
#start=240; rot=0.72; gamp=1.086 #maxxhue purple fire
#start=245; rot=0.70; gamp=1.084 #maxhue purple fire
#start=240; rot=0.72; gamp=1.25 #oil fire
#start=180; rot=0.96; gamp=1.
#start=180; rot=1.; gamp=1.25 #chemical fire
#start=150; rot=1; gamp=2 #another oil fire
#start=150; rot=1; gamp=1.25 #
#start=285; rot=0.54
#start=285; rot=0.54; gamp=1.25 #slightly purple fire
#start=285; rot=0.54; gamp=1.6 #yet another furnace

####manual extended param modifiers
#pwr_amp=0.625; eps_amp=0.05; eps_pwr=2
#pwr_amp=0.5; eps_amp=0.05; eps_pwr=2
#sw_sc=-0.5; pwr_sc=2
#k_ellipse=0.75; alpha_ellipse=120.; insectoid=True
minLight=1.e-15; maxLight=1
ColorModel=2
#trimWhite=False

cfitmode = 1 #0:off, 1:fit for color gradient display, 2:fit for curves & output map, too

minSat=1.0
maxsat=1.0
scalsat = 1

cliprange = [0.1, 0.9]

tintMode = 1
startRGB = [0., 0., 1.5]; startPwr = 4.0
endRGB = [1., 1., 1.]; endPwr = 2.0
#allow for triple powers
#startPwr = [8., 8., 8.]; endPwr = [2., 2., 2.]
whtPwr = 4.
trimWhite=True

iclipmin = int(cliprange[0]*(nlev-1)+0.5)
iclipmax = int(cliprange[1]*(nlev-1)+0.5)
print('iclipmin = ', iclipmin)
print('iclipmax = ', iclipmax)

endcolor = start+360*rot
endcolor -= 360.*np.floor(endcolor/360.)
print('End color angle = ', endcolor)

cx = optimize_gamma.cmap(gamit=gamit, do_iter=do_iter, start=start, rot=rot, gstart=gstart, gamp=gamp, grot=grot,
                         pwr_amp=pwr_amp, eps_amp=eps_amp, eps_pwr=eps_pwr, sw_sc=sw_sc, pwr_sc=pwr_sc,
                         k_ellipse=k_ellipse, alpha_ellipse=alpha_ellipse, insectoid=insectoid,
                         reverse=reverse, cuberot=cuberot, nlev=nlev, minSat=minSat, maxSat=maxSat,
                         scalsat=scalsat, minLight=minLight, maxLight=maxLight, ColorModel=ColorModel,
                         iterSat=iterSat, epsSat=epsSat, rgbebs=rgbeps, gameps=gameps,
                         cliprange=cliprange, wluma=wluma, trimWhite=trimWhite, verbose=verbose,
                         tintMode=tintMode, startRGB=startRGB, endRGB=endRGB, startPwr=startPwr, endPwr=endPwr,
                         whtPwr=whtPwr)

if cfitmode == 2: cx = fcc.cmap(cx)

fract = np.linspace(0, 1, nlev)
def plot_testimage(cx, cfitmode):
    n = 100000
    m = 100
    r = 3
    x = np.random.randn(n)
    y = np.random.randn(n)
    fig, ax = plt.subplots(figsize=(6, 6), constrained_layout=True)
    ax.set_xlim([-r,r])
    ax.set_ylim([-r,r])
    ax.set_axis_off()
    if cfitmode == 1: cx = fcc.cmap(cx)
    gradient = np.vstack((fract, fract))
    plt.hexbin(x, y, gridsize=m, cmap=cx)
    plt.show(block=False)

def plot_combined(cx, nlevgrad, cfitmode):
    wluma=[0.299, 0.587, 0.114, 0]
    #wluma = [0.2126, 0.7152, 0.0722, 0]
    fract = np.linspace(0, 1, nlevgrad)
    rgba = cx(fract)
    srgb = cx(fract)[:,:3]
    Lab = cspace_converter("sRGB1", "CAM02-UCS")(srgb)
    L = Lab[:,0]
    a = Lab[:,1]
    b = Lab[:,2]
    C = np.sqrt(a**2+b**2)
    Y = np.dot(rgba, wluma)
    fig, axes = plt.subplots(figsize=(8, 8), nrows=2, constrained_layout=True)
    ax0 = axes[0]
    ax1 = axes[1]
    ax0.plot(fract, 0.01*L, color='k', linestyle='--', alpha=0.5)
    ax0.plot(fract, Y, color='k', linestyle='-', alpha=1)
    ax0.plot(fract, fract, color='k', linestyle=':', alpha=0.5)
    ax0.plot(fract, 0.01*C, color='y', linestyle='--', alpha=1)
    col = ['r', 'g', 'b']
    for i in range(3):
        ax0.plot(fract, rgba[:, i], color=col[i])
    ax0.set_xlabel('index')
    ax0.set_ylabel('RGB')
    ax0.set_xlim([0,1])
    ax0.set_ylim([0,1])
    ax0.set_xticks(np.arange(0, 1, step=0.1))
    ax0.set_yticks(np.arange(0, 1, step=0.1))
    ax0.minorticks_on()
    if cfitmode == 1: cx = fcc.cmap(cx)
    gradient = np.vstack((fract, fract))
    ax1.imshow(gradient, aspect='auto', cmap=cx)
    #ax1.set_axis_off()
    ax1.get_xaxis().set_visible(False)
    ax1.get_yaxis().set_visible(False)
    ax0.grid(True, which='major', color='#808080')
    ax0.grid(True, which='minor', color='#e0e0e0')
    plt.show()

def plot_rgbhelix(cx, nlevgrad):
    fract = np.linspace(0, 1, nlevgrad)
    rgba = cx(fract)
    srgb = rgba[:,:3]
    rr = rgba[:,0]
    gg = rgba[:,1]
    bb = rgba[:,2]
    Lab = cspace_converter("sRGB1", "CAM02-UCS")(srgb)
    L = Lab[:,0]
    a = Lab[:,1]
    b = Lab[:,2]
    fig = plt.figure(figsize=(8, 8))
    ax = fig.gca(projection='3d')
    ax.set_xlabel('r')
    ax.set_ylabel('g')
    ax.set_zlabel('b')
    ax.set_xlim([0,1])
    ax.set_ylim([0,1])
    ax.set_zlim([0,1])
    ax.plot(rr, gg, bb)
    plt.show()
    
def plot_labhelix(cx, nlevgrad):
    fract = np.linspace(0, 1, nlevgrad)
    rgba = cx(fract)
    srgb = rgba[:,:3]
    rr = rgba[:,0]
    gg = rgba[:,1]
    bb = rgba[:,2]
    Lab = cspace_converter("sRGB1", "CAM02-UCS")(srgb)
    L = Lab[:,0]
    a = Lab[:,1]
    b = Lab[:,2]
    fig = plt.figure(figsize=(8, 8))
    ax = fig.gca(projection='3d')
    ax.set_xlabel('a')
    ax.set_ylabel('b')
    ax.set_zlabel('L')
    ax.set_xlim([-45,45])
    ax.set_ylim([-45,45])
    ax.set_zlim([0,100])
    ax.plot(a, b, L)
    plt.show()

if write_gpmap:
    fgp = open('x.rgba', 'w')
    fgp.write('# {:}\n'.format(cx.name))
    fgp.write('# gamit={:d}; do_iter={:}\n'.format(gamit, do_iter))
    fgp.write('# start={:.3f}; rot={:.6f}\n'.format(start, rot))
    fgp.write('# scalsat={:.3f}; gstart={:.3f}\n'.format(scalsat, gstart))
    fgp.write('# gamma={:.6f}; gamp={:.3f}; grot={:.3f}\n'.format(optimize_gamma.gamma_export, gamp, grot))
    fgp.write('# nlev={:d}; ColorModel={:d}; reverse={:}; cuberot={:}\n'.format(nlev, ColorModel, reverse, cuberot))
    fgp.write('# minLight={:.3f}; maxLight={:.3f}; trimWhite={:}\n'.format(minLight, maxLight, trimWhite))
    fgp.write('# pwr_amp={:.3f}; eps_amp={:.3f}; eps_pwr={:.3f}\n'.format(pwr_amp, eps_amp, eps_pwr))
    fgp.write('# sw_sc={:.3f}; pwr_sc={:.3f}\n'.format(sw_sc, pwr_sc))
    fgp.write('# k_ellipse={:.3f}; alpha_ellipse={:.3f}; insectoid={:}\n'.format(k_ellipse, alpha_ellipse, insectoid)) #k_ellipse=k_ellipse, alpha_ellipse=alpha_ellipse, insectoid=insectoid,
    fgp.write('# iterSat={:d}; epsSat={:.3e}; rgbeps={:.3e}; gameps={:.3e}\n'.format(iterSat, epsSat, rgbeps, gameps))
    fgp.write('# cliprange={:}; wluma={:}\n'.format(cliprange, wluma))
    if tintMode:
        fgp.write('# startRGB={:}; endRGB={:}\n'.format(startRGB, endRGB))
        fgp.write('# startPwr={:}; endPwr={:}; whtPwr={:}\n'.format(startPwr, endPwr, whtPwr))
    for i in range(nlev):
        x = i/(nlev-1)
        outline = '{:.6f}  {:.6f} {:.6f} {:.6f} {:.6f}'.format(x, *cx(x))
        #print(outline)
        fgp.write(outline+'\n')
    fgp.close()

if show_test_image:
    #plot_testimage(cx, cfitmode)
    plot_combined(cx, nlev, cfitmode)
    #plot_rgbhelix(cx, nlev)
    #plot_labhelix(cx, nlev)
